@@ -1,15 +1,79 @@
+"""This module contains functions that execute assistants' dependencies and run
+sections. These functions usually assume that their input has been previously
+checked by `devassistant.yaml_checker.check`."""
+import os
 import re
 import shlex
+import string
 import sys
 
-from devassistant import command
+import six
+
 from devassistant import exceptions
 from devassistant.logger import logger
 from devassistant import package_managers
 from devassistant import settings
+from devassistant import utils
+
+
+class Command(object):
+    """A class that represents a Yaml command. It has these members:
+
+    - comm_type: type of command
+    - comm: command input as it was specified *literally* (without substitution of vars, etc)
+    - input_log_res: logical result of command input
+    - input_res: result of command input
+    - kwargs: global context taken at point of execution of this command
+    """
+
+    command_runners = None
+
+    @classmethod
+    def load_command_runners(cls):
+        if not cls.command_runners:
+            cls.command_runners = utils.import_module('devassistant.command_runners')
+        return cls.command_runners
+
+    def __init__(self, comm_type, comm, kwargs={}):
+        self.comm_type = comm_type
+        self.comm = comm
+        self.had_exec_flag = False
+        if comm_type.endswith('~'):
+            self.comm_type = self.comm_type[:-1]
+            self.had_exec_flag = True
+        self._input_log_res = None
+        self._input_res = None
+        self.files_dir = kwargs.get('__files_dir__', [''])[-1]
+        self.files = kwargs.get('__files__', [''])[-1]
+        self.kwargs = kwargs
+
+    def run(self):
+        for cr in type(self).load_command_runners().command_runners:
+            if cr.matches(self):
+                return cr.run(self)
+
+        raise exceptions.CommandException('No runner for command "{ct}: {c}".'.\
+            format(ct=self.comm_type, c=self.input_res))
+
+    @property
+    def input_log_res(self):
+        return self._eval_input()[0]
+
+    @property
+    def input_res(self):
+        return self._eval_input()[1]
+
+    def _eval_input(self):
+        if self._input_log_res is None:
+            runner = self.kwargs.get('__assistant__', None)
+            if self.had_exec_flag:
+                method = eval_exec_section
+            else:
+                method = eval_literal_section
+            self._input_log_res, self._input_res = method(self.comm, self.kwargs, runner)
+
+        return self._input_log_res, self._input_res
 
-if sys.version_info[0] > 2:
-    basestring = str
 
 def dependencies_section(section, kwargs, runner=None):
     # "deps" is the same structure as gets returned by "dependencies" method
@@ -23,14 +87,17 @@ def dependencies_section(section, kwargs, runner=None):
             # rpm dependencies (can't handle anything else yet)
             # we don't allow general commands, only "call"/"use" command here
             if dep_type in ['call', 'use']:
-                deps.extend(command.Command(dep_type, dep_list, kwargs).run())
-            elif dep_type in package_managers.managers.keys(): # handle known types of deps the same, just by appending to "deps" list
-                deps.append({dep_type: dep_list})
+                deps.extend(Command(dep_type, dep_list, kwargs).run())
+            # handle known types of deps the same, just by appending to "deps" list
+            elif dep_type in package_managers.managers.keys():
+                fmtd = list(map(lambda dep: format_str(dep, kwargs), dep_list))
+                deps.append({dep_type: fmtd})
             elif dep_type.startswith('if'):
                 possible_else = None
-                if len(section) > i + 1: # do we have "else" clause?
+                if len(section) > i + 1:  # do we have "else" clause?
                     possible_else = list(section[i + 1].items())[0]
-                _, skip_else, to_run = get_section_from_condition((dep_type, dep_list), possible_else, kwargs)
+                _, skip_else, to_run = get_section_from_condition((dep_type, dep_list),
+                                                                  possible_else, kwargs)
                 if to_run:
                     deps.extend(dependencies_section(to_run, kwargs, runner=runner))
             elif dep_type == 'else':
@@ -45,22 +112,47 @@ def dependencies_section(section, kwargs, runner=None):
 
     return deps
 
-def run_section(section, kwargs, runner=None):
+
+def expand_dependencies_section(section, kwargs):
+    """Expands dependency section, e.g. substitues "use: foo" for its contents, but
+    doesn't evaluate conditions nor substitue variables."""
+    deps = []
+
+    for dep in section:
+        for dep_type, dep_list in dep.items():
+            if dep_type in ['call', 'use']:
+                deps.extend(Command(dep_type, dep_list, kwargs).run())
+            elif dep_type.startswith('if ') or dep_type == 'else':
+                deps.append({dep_type: expand_dependencies_section(dep_list, kwargs)})
+            else:
+                deps.append({dep_type: dep_list})
+
+    return deps
+
+
+def run_section(section, kwargs=None, runner=None):
+    if kwargs is None:
+        kwargs = {}
+    return eval_exec_section(section, kwargs, runner)
+
+
+def eval_exec_section(section, kwargs, runner=None):
     skip_else = False
+    retval = [False, '']
+
+    if isinstance(section, six.string_types):
+        return evaluate_expression(section, kwargs)
 
     for i, command_dict in enumerate(section):
         if getattr(runner, 'stop_flag', False):
             break
         for comm_type, comm in command_dict.items():
-            retval = [False, '']
-            if comm_type.startswith('$'):
-                # intentionally pass kwargs as dict, not as keywords
-                retval = assign_variable(comm_type, comm, kwargs)
-            elif comm_type.startswith('if'):
+            if comm_type.startswith('if'):
                 possible_else = None
-                if len(section) > i + 1: # do we have "else" clause?
+                if len(section) > i + 1:  # do we have "else" clause?
                     possible_else = list(section[i + 1].items())[0]
-                _, skip_else, to_run = get_section_from_condition((comm_type, comm), possible_else, kwargs)
+                _, skip_else, to_run = get_section_from_condition((comm_type, comm),
+                                                                  possible_else, kwargs)
                 # run with original kwargs, so that they might be changed for code after this
                 if to_run:
                     retval = run_section(to_run, kwargs, runner=runner)
@@ -69,9 +161,10 @@ def run_section(section, kwargs, runner=None):
                     msg = 'Yaml error: encountered "else" with no associated "if", skipping.'
                     raise exceptions.YamlSyntaxError(msg)
                 skip_else = False
-            elif comm_type.startswith('for'):
+            elif comm_type.startswith('for '):
                 # syntax: "for $i in $x: <section> or "for $i in cl_command: <section>"
-                control_vars, eval_expression = get_for_control_var_and_eval_expr(comm_type, kwargs)
+                control_vars, eval_expression = get_for_control_var_and_eval_expr(comm_type,
+                                                                                  kwargs)
                 for i in eval_expression:
                     if len(control_vars) == 2:
                         kwargs[control_vars[0]] = i[0]
@@ -79,43 +172,99 @@ def run_section(section, kwargs, runner=None):
                     else:
                         kwargs[control_vars[0]] = i
                     retval = run_section(comm, kwargs, runner=runner)
+            elif comm_type.startswith('$'):
+                # commands that can have exec flag appended follow
+                if comm_type.endswith('~'):  # on exec flag, eval comm as exec section
+                    comm_ret = eval_exec_section(comm, kwargs, runner)
+                else:  # with no exec flag, eval comm as input section
+                    comm_ret = eval_literal_section(comm, kwargs, runner)
+                retval = assign_variable(comm_type, *comm_ret, kwargs=kwargs)
             else:
-                retval = command.Command(comm_type,
-                                         comm,
-                                         kwargs).run()
+                retval = Command(comm_type, comm, kwargs=kwargs).run()
 
-            if not isinstance(retval, (list, tuple)):
+            if not isinstance(retval, (list, tuple)) or len(retval) != 2:
                 raise exceptions.RunException('Bad return value of last command ({ct}: {c}): {r}'.\
                         format(ct=comm_type, c=comm, r=retval))
             assign_last_result(kwargs, *retval)
 
-    return [kwargs.get(settings.LAST_LR_VAR, False), kwargs.get(settings.LAST_R_VAR, '')]
+    return retval
+
+
+def eval_literal_section(section, kwargs, runner=None):
+    retval = (False, '')
+
+    if isinstance(section, six.string_types):
+        # two ~~ are an escape sequence (we want to start the string by "~", not
+        #  eval it exec section
+        eval_exec = False
+        if section.startswith('~~'):
+            section = section[1:]
+        elif section.startswith('~'):
+            eval_exec = True
+
+        if eval_exec:
+            retval = eval_exec_section(section[1:], kwargs, runner)
+        elif is_var(section):
+            # if it is a defined variable, return it as it is, not necessarily as string
+            # if it is a variable, but not defined, return it's name
+            varname = get_var_name(section)
+            res = kwargs.get(varname, section)
+            retval = (bool(res), res)
+        else:
+            res = format_str(section, kwargs)
+            retval = (bool(res), res)
+    elif isinstance(section, list):
+        retlist = []
+        for item in section:
+            # TODO: check for stop flag
+            retlist.append(eval_literal_section(item, kwargs)[1])
+        retval = (bool(retlist), retlist)
+    elif isinstance(section, dict):
+        retdict = {}
+        for k, v in section.items():
+            k = format_str(k, kwargs)
+            # TODO: check for stop flag
+            if k.endswith('~'):
+                retdict[k[:-1]] = eval_exec_section(v, kwargs, runner)[1]
+            else:
+                retdict[k] = eval_literal_section(v, kwargs, runner)[1]
+        retval = (bool(retdict), retdict)
+    elif isinstance(section, bool):
+        retval = (section, section)
+
+    return retval
+
 
 def assign_last_result(kwargs, log_res, res):
     kwargs[settings.LAST_LR_VAR] = log_res
     kwargs[settings.LAST_R_VAR] = res
 
+
 def parse_for(control_line):
-    """Returns name of loop control variable(s) and expression to iterate on.
+    """Returns name of loop control variable(s), iteration type (in/word_in) and
+    expression to iterate on.
 
     For example:
     - given "for $i in $foo", returns (['i'], '$foo')
-    - given "for ${i} in $(ls $foo)", returns (['i'], 'ls $foo')
+    - given "for ${i} in $(ls $foo)", returns (['i'], '$(ls $foo)')
     - given "for $k, $v in $foo", returns (['k', 'v'], '$foo')
     """
     error = 'For loop call must be in form \'for $var in expression\', got: ' + control_line
-    regex = re.compile(r'for\s+(\${?\S}?)(?:\s*,\s+(\${?\S}?))?\s+in\s+(\S.+)')
-    res = regex.match(control_line).groups()
+    regex = re.compile(r'for\s+(\${?\S+}?)(?:\s*,\s+(\${?\S+}?))?\s+(in|word_in)\s+(\S.+)')
+    res = regex.match(control_line)
     if not res:
         raise exceptions.YamlSyntaxError(error)
 
+    groups = res.groups()
     control_vars = []
-    control_vars.append(get_var_name(res[0]))
-    if res[1]:
-        control_vars.append(get_var_name(res[1]))
-    expr = res[2]
+    control_vars.append(get_var_name(groups[0]))
+    if groups[1]:
+        control_vars.append(get_var_name(groups[1]))
+    iter_type = groups[2]
+    expr = groups[3]
+
+    return (control_vars, iter_type, expr)
 
-    return (control_vars, expr)
 
 def get_for_control_var_and_eval_expr(comm_type, kwargs):
     """Returns tuple that consists of control variable name and iterable that is result
@@ -126,7 +275,7 @@ def get_for_control_var_and_eval_expr(comm_type, kwargs):
     - given 'for $i, $j in $foo' it returns (['i', 'j'], [('foo', 'bar')])
     """
     # let possible exceptions bubble up
-    control_vars, expression = parse_for(comm_type)
+    control_vars, iter_type, expression = parse_for(comm_type)
     eval_expression = evaluate_expression(expression, kwargs)[1]
 
     iterval = []
@@ -136,10 +285,14 @@ def get_for_control_var_and_eval_expr(comm_type, kwargs):
                     format(t=type(eval_expression)))
         else:
             iterval = list(eval_expression.items())
-    elif isinstance(eval_expression, basestring):
-        iterval = eval_expression.split()
+    elif isinstance(eval_expression, six.string_types):
+        if iter_type == 'word_in':
+            iterval = eval_expression.split()
+        else:
+            iterval = eval_expression
     return control_vars, iterval
 
+
 def get_section_from_condition(if_section, else_section, kwargs):
     """Returns section that should be used from given if/else sections by evaluating given
     condition.
@@ -164,42 +317,59 @@ def get_section_from_condition(if_section, else_section, kwargs):
     else:
         return (1, skip, else_section[1]) if skip else (1, skip, None)
 
-def assign_variable(variable, comm, kwargs):
-    """Assigns *result* of expression to variable. If there are two variables separated by
-    comma, the first gets assigned *logical result* and the second the *result*.
-    The variable is then put into kwargs (overwriting original value, if already there).
-    Note, that unlike other methods, this method has to accept kwargs, not **kwargs.
 
-    Even if comm has *logical result* == False, output is still stored and
-    this method doesn't fail.
+def assign_variable(variable, log_res, res, kwargs):
+    """Assigns given result (resp. logical result and result) to a variable
+    (resp. to two variables). log_res and res are already computed result
+    of an exec/input section. For example:
+
+    $foo~: $spam and $eggs
+    $log_res, $foo~: $spam and $eggs
+
+    $foo:
+      some: struct
+    $log_res, $foo:
+      some:
+        other:
+          struct
 
     Args:
         variable: variable (or two variables separated by ",") to assign to
-        comm: either another variable or command to run
+        log_res: logical result of evaluated section
+        res: result of evaluated section
+
+    Raises:
+        YamlSyntaxError: if there are more than two variables
     """
+    if variable.endswith('~'):
+        variable = variable[:-1]
     comma_count = variable.count(',')
     if comma_count > 1:
         raise exceptions.YamlSyntaxError('Max two variables allowed on left side.')
 
-    res1, res2 = evaluate_expression(comm, kwargs)
     if comma_count == 1:
         var1, var2 = map(lambda v: get_var_name(v), variable.split(','))
-        kwargs[var1] = res1
+        kwargs[var1] = log_res
     else:
         var2 = get_var_name(variable)
-    kwargs[var2] = res2
-    return res1, res2
+    kwargs[var2] = res
+    return log_res, res
+
+
+_var_matcher = re.compile(r'^\s*\$\{?([\w]+)\}?\s*$')
+
 
 def is_var(string):
-    return string.startswith('$')
+    return bool(_var_matcher.match(string))
+
+
+def get_var_name(dollar_variable):
+    name = dollar_variable.strip('"\'')
+    matched = _var_matcher.match(name)
+    if not matched:
+        raise exceptions.YamlSyntaxError('Not a proper variable name: ' + dollar_variable)
+    return matched.group(1)
 
-def get_var_name(dolar_variable):
-    name = dolar_variable.strip()
-    name = name.strip('"\'')
-    if not name.startswith('$'):
-        raise exceptions.YamlSyntaxError('Not a proper variable name: ' + dolar_variable)
-    name = name[1:] # strip the dollar
-    return name.strip('{}')
 
 ### Expression evaluation
 class Interpreter(object):
@@ -224,6 +394,7 @@ class Interpreter(object):
 
         # The tokenizer considers all tokens in "$()" to be literals
         self.in_shell = False
+        self.as_root = False
 
     class symbol_base(object):
         id = None
@@ -278,10 +449,10 @@ class Interpreter(object):
     def tokenize(self, program):
         self.in_shell = False
         lexer = shlex.shlex(program)
-        lexer.wordchars += "$-/\\."
+        lexer.wordchars += "$-/\\.:`={}"
 
         for tok in lexer:
-            if tok in ["and", "or", "not", "defined", "(", ")", "in", "$"]:
+            if tok in ["and", "or", "not", "defined", "(", ")", "in", "$", "as_root"]:
                 # operators
                 symbol = self.symbol_table.get(tok)
                 yield symbol()
@@ -344,6 +515,7 @@ def evaluate_expression(expression, names):
     interpr.symbol("in", 10)
     interpr.symbol("defined", 10)
     interpr.symbol("$", 10)
+    interpr.symbol("as_root", 10)
     interpr.symbol("(name)")
     interpr.symbol("(literal)")
     interpr.symbol("(end)")
@@ -368,7 +540,7 @@ def evaluate_expression(expression, names):
         # If there is a known variable in the literal, substitute it for its
         # value
         for v in reversed(sorted(interpr.names.keys())):
-            self.value = self.value.replace("$" + v, str(interpr.names[v]))
+            self.value = self.value.replace("$" + v, six.text_type(interpr.names[v]))
         # if self.value is in double/single quotes, strip them (but only the outer quotes)
         ret = self.value
         if ret.startswith('"'):
@@ -435,24 +607,40 @@ def evaluate_expression(expression, names):
             while 1:
                 if interpr.token.id == ")":
                     break
+                # if there is (name), tokenizer has already stripped
+                # the "$", but we need to keep it for below substitution
+                if interpr.token.id == "(name)":
+                    interpr.token.value = "$" + interpr.token.value
                 cmd.append(interpr.token.value)
                 interpr.advance()
 
         cmd = " ".join(cmd)
 
         # Substitute the variables
-        for v in reversed(sorted(interpr.names.keys())):
-            cmd = cmd.replace("$" + v, str(interpr.names[v]))
+        formatted_cmd = format_str(cmd, interpr.names)
 
         success = True
+        exec_mode = 'cl_r' if interpr.as_root else 'cl'
         try:
-            output = command.Command('cl_n', cmd, interpr.names).run()[1]
+            output = Command(exec_mode, cmd, interpr.names).run()[1]
         except exceptions.RunException as ex:
             success = False
             output = ex.output
 
         interpr.advance(")")
         interpr.in_shell = False
+        interpr.as_root = False
+
+        return success, output
+
+    @interpr.method("as_root")
+    def nud(self):
+        interpr.as_root = True
+        right = interpr.expression(10)
+        interpr.as_root = False
+
+        success = bool(right[0])
+        output = right[1]
 
         return success, output
 
@@ -470,3 +658,55 @@ def evaluate_expression(expression, names):
 
     # With the language defined, evaluate the expression
     return interpr.parse(expression)
+
+
+# spliting strings by _command_splitter.findall(str) preserves whitespace
+_command_splitter = re.compile(r'(\s+|\S+)')
+# we want to do homedir expansion in quotes (which bash doesn't)
+_homedir_matcher = re.compile('\\\\*~')
+
+
+def _homedir_expand(matchobj):
+    # therefore we must hack around this here
+    if len(matchobj.group(0)) % 2 == 0:
+        # even length => odd number of backslashes => eat one and don't expand
+        return matchobj.group(0)[:-2] + '~'
+    else:
+        # odd length => even number of backslashes => expand an
+        return matchobj.group(0)[:-1] + os.path.expanduser('~')
+
+
+def format_str(s, kwargs):
+    files_dir = kwargs.get('__files_dir__', [''])[-1]
+    files = kwargs.get('__files__', [{}])[-1]
+    # If command is false/true in yaml file, it gets converted to False/True
+    # which is bool object => convert
+    if isinstance(s, bool):
+        comm = str(s).lower()
+    else:
+        comm = s
+
+    new_comm = []
+    parts_list = _command_splitter.findall(comm)
+
+    # replace parts that match something from _files (can be either name
+    # if "&" didn't expand in yaml; or the dict if "&" did expand)
+    for c in parts_list:
+        if isinstance(c, dict):
+            # TODO: raise a proper error if c['source'] is not present
+            new_comm.append(os.path.join(files_dir, c['source']))
+        elif c.startswith('*'):
+            c_file = c[1:].strip('{}')
+            if c_file in files:
+                new_comm.append(os.path.join(files_dir, files[c_file]['source']))
+            else:
+                new_comm.append(c)
+        else:
+            new_comm.append(c)
+
+    new_comm = ''.join(new_comm)
+
+    # substitute cli arguments for their values
+    substituted = string.Template(new_comm).safe_substitute(kwargs)
+
+    return _homedir_matcher.sub(_homedir_expand, substituted)
